@@ -2,89 +2,113 @@
 # Copyright 2016 Roger Marsh
 # Licence: See LICENCE (BSD licence)
 
-"""Edit or insert dialogue for Chess Query Language (ChessQL) statement record.
+"""Customise edit toplevel to edit or insert Chess Query Language (ChessQL)
+statement record.
+
+ChessQL statements obey the syntax published for CQL version 6.0.1 (by Gady
+Costeff).
+
 """
-import tkinter
 import tkinter.messagebox
 
 from solentware_grid.gui.dataedit import DataEdit
 from solentware_misc.gui.exceptionhandler import ExceptionHandler
 
-from .cqldisplay import DialogueCQLDisplay, DialogueCQLEdit
+from .cqltoplevel import CQLToplevel, CQLToplevelEdit
+from .topleveltext import EditText
 
 
-class ChessDBeditCQL(ExceptionHandler, DataEdit):
-    """Dialog to edit a ChessQL statement on, or insert one into, database.
+class CQLDbEdit(ExceptionHandler, EditText, DataEdit):
+    """Edit ChessQL statement on database, or insert a new record.
 
-    The ChessQL statement is in it's own Toplevel widget.
+    parent is used as the master argument in CQLToplevel calls.
+
+    ui is used as the ui argument in CQLToplevel calls.
+
+    newobject, parent, oldobject, and the one or two CQLToplevel instances
+    created, are used as arguments in the super.__init__ call.
+
+    showinitial determines whether a CQLToplevel is created for oldobject if
+    there is one.
+
+    Attribute text_name provides the name used in widget titles and message
+    text.
+
+    Methods get_title_for_object and set_item, and properties ui_base_table;
+    ui_items_in_toplevels; and ui, allow similar methods in various classes
+    to be expressed identically and defined once.
 
     """
+    text_name = 'ChessQL Statement'
 
     def __init__(self, newobject, parent, oldobject, showinitial=True, ui=None):
-        """Extend and create dialogue to edit or insert ChessQL statement."""
-        if oldobject:
-            title = ':  '.join((
-                'Edit ChessQL statement',
-                oldobject.value.get_name_text()))
-        else:
-            title = 'Insert ChessQL Statement'
+        """Extend and create toplevel to edit or insert ChessQL statement."""
+        if not oldobject:
             showinitial = False
-        self.__title = title.split(':')[0]
-        if showinitial:
-            showinitial = DialogueCQLDisplay(master=parent, ui=ui)
-            if ui is not None:
-                ui.partials_in_toplevels.add(showinitial)
-            showinitial.cql_statement.process_statement(
-                oldobject.get_srvalue())
-            showinitial.set_cql_statement()
-        newview = DialogueCQLEdit(master=parent, ui=ui)
-        if ui is not None:
-            ui.partials_in_toplevels.add(newview)
-        newview.cql_statement.process_statement(newobject.get_srvalue())
-        newview.set_cql_statement()
         super().__init__(
             newobject,
             parent,
             oldobject,
-            newview,
-            title,
-            oldview=showinitial,
+            CQLToplevelEdit(master=parent, ui=ui),
+            '',
+            oldview=CQLToplevel(master=parent,
+                                ui=ui) if showinitial else showinitial,
             )
+        self.initialize()
 
-        # Bind only to newview.score because it alone takes focus.
-        self.bind_buttons_to_widget(newview.score)
+    def get_title_for_object(self, object_=None):
+        """Return title for Toplevel containing a ChessQL statement object_.
 
-        self.ui = ui
+        Default value of object_ is oldobject attribute from DataEdit class.
+
+        """
+        if object_ is None:
+            object_ = self.oldobject
+        if object_:
+            return '  '.join((
+                self.text_name.join(('Edit ', ':')),
+                object_.value.get_name_text()))
+        else:
+            return ''.join(('Insert ', self.text_name))
+
+    @property
+    def ui_base_table(self):
+        return self.ui.base_partials
+
+    @property
+    def ui_items_in_toplevels(self):
+        return self.ui.partials_in_toplevels
+
+    @property
+    def ui(self):
+        return self.newview.ui
+
+    def set_item(self, view, object_):
+        view.cql_statement.process_statement(object_.get_srvalue())
+        view.set_and_tag_item_text(reset_undo=True)
         
     def dialog_ok(self):
-        """Update record and return update action response (True for updated).
+        """Return update action response (True for deleted).
 
-        Check that database is open and is same one as update action was
-        started.
+        Delegate to superclass if the active item on the main display is not a
+        'Position | Partial' widget and contains a valid statement.
 
         """
-        if self.ui.database is None:
-            self.status.configure(
-                text='Cannot update because not connected to a database')
-            if self.ok:
-                self.ok.destroy()
-                self.ok = None
-            self.blockchange = True
-            return False
         self.newobject.value.load(
             repr(self.newview.get_name_cql_statement_text()))
+        title = self.get_title_for_object()
         if not len(self.newobject.value.get_name_text()):
             if not self.newobject.value.cql_error:
                 tkinter.messagebox.showerror(
                     parent=self.parent,
-                    title=self.__title,
+                    title=title,
                     message=''.join((
                         "The ChessQL statement has no name.\n\nPlease enter ",
                         "it's name as the first line of text.'")))
             else:
                 tkinter.messagebox.showerror(
                     parent=self.parent,
-                    title=self.__title,
+                    title=title,
                     message=''.join((
                         "The text does not contain a valid ChessQL ",
                         "statement. ")))
@@ -92,7 +116,7 @@ class ChessDBeditCQL(ExceptionHandler, DataEdit):
         elif self.newobject.value.cql_error:
             if tkinter.messagebox.YES != tkinter.messagebox.askquestion(
                 parent=self.parent,
-                title=self.__title,
+                title=title,
                 message=''.join((
                     'Confirm request to update ChessQL statement named:\n\n',
                     self.newobject.value.get_name_text(),
@@ -103,7 +127,7 @@ class ChessDBeditCQL(ExceptionHandler, DataEdit):
             if self.ui.partial_items.active_item.sourceobject is None:
                 tkinter.messagebox.showinfo(
                     parent=self.parent,
-                    title=self.__title,
+                    title=title,
                     message=''.join((
                         "Cannot use this insert dialogue while the active ",
                         "item in cql queries is one opened by menu action ",
@@ -111,20 +135,8 @@ class ChessDBeditCQL(ExceptionHandler, DataEdit):
                 return False
         return super().dialog_ok()
 
-    def tidy_on_destroy(self):
-        """Clear up after dialogue destruction."""
-        self.ui.partials_in_toplevels.discard(self.oldview)
-        self.ui.partials_in_toplevels.discard(self.newview)
-        self.ui.base_partials.selection.clear()
-
     def edit(self, commit=True):
-        """Edit the records and refresh widgets.
-
-        The ChessQL query statement update is delegated to the superclass.
-
-        This method updates the foundset calculated from the query.
-
-        """
+        """Delegate to superclass to edit record then update game list."""
         if commit:
             self.datasource.dbhome.start_transaction()
         super().edit(commit=False)
@@ -138,13 +150,7 @@ class ChessDBeditCQL(ExceptionHandler, DataEdit):
             self.datasource.dbhome.commit()
         
     def put(self, commit=True):
-        """Insert the records and refresh widgets.
-
-        The ChessQL query statement update is delegated to the superclass.
-
-        This method updates the foundset calculated from the query.
-
-        """
+        """Delegate to superclass to insert record then insert game list."""
         if commit:
             self.datasource.dbhome.start_transaction()
         super().put(commit=False)
