@@ -4,29 +4,21 @@
 
 """Widgets to display and edit Chess Query Language (ChessQL) statements.
 
+ChessQL statements obey the syntax published for CQL version 6.0.1 (by Gady
+Costeff).
+
 These four classes display ChessQL statements in the main window: they are used
-in the querygrid module.
+in the cqlgrid module.
 
-The CQLDisplay class binds events to navigate between widgets.
+The _CQLDisplay class provides attributes and behaviour shared by the
+CQLDisplay, CQLDisplayInsert, and CQLDisplayEdit, classes.  It also provides
+properties to support implementation of behaviour shared with the Game*,
+Repertoire*, and Query*, classes.
 
-The DatabaseCQLDisplay class adds delete record to the CQLDisplay class.
-
-The DatabaseCQLInsert class adds insert record to the DatabaseCQLDisplay
-class but does not bind delete record to any events.
-
-The DatabaseCQLEdit class adds edit record to the DatabaseCQLInsert class
-but does not bind delete record to any events.
-
-These three classes display games in their own Toplevel widget: they are used
-in the gamedbdelete, gamedbedit, and gamedbshow, modules.
-
-The CQLDialogue class binds events to navigate between widgets.
-
-The DialogueCQLDisplay class adds insert and delete record to the
-CQLDialogue class.
-
-The DialogueCQLEdit class adds insert and edit record to the CQLDialogue
-class.
+The CQLDisplay, CQLDisplayInsert, and CQLDisplayEdit, classes are subclasses
+of the relevant ShowText, InsertText, EditText, DisplayText, and ListGamesText,
+classes from the displaytext module; to implement behaviour shared with all
+Text widgets in the main display (that includes widgets displaying PGN text).
 
 """
 
@@ -45,46 +37,43 @@ from ..core.chessrecord import ChessDBrecordPartial
 from ..core.cqlstatement import CQLStatement
 from .eventspec import EventSpec
 from .display import Display
-from .. import APPLICATION_NAME
+from .displaytext import (
+    ShowText, InsertText, EditText, DisplayText, ListGamesText,)
 
 
-class CQLDisplay(ExceptionHandler, Display):
+class _CQLDisplay(ExceptionHandler, Display):
     
-    """Manage UI interaction with database for displayed ChessQL statements.
+    """Extend and link ChessQL statement to database.
 
-    CQLDisplay is a subclass of DataNotify so that modifications to
-    the database record outside an instance prevent database update using
-    the instance.  This class provides methods to update the database from
-    the instance; and to switch to other displayed ChessQL statements.
+    sourceobject - link to database.
+
+    Attribute binding_labels specifies the order navigation bindings appear
+    in popup menus.
+
+    Method insert_item_database allows records to be inserted into a database
+    from any CQL widget.
 
-    Subclasses provide the widget to display the ChessQL statements.
-    
     """
 
-    binding_labels = tuple(
-        [b[1:] for b in (
-            EventSpec.partialdisplay_to_position_grid,
-            EventSpec.partialdisplay_to_active_game,
-            EventSpec.partialdisplay_to_game_grid,
-            EventSpec.partialdisplay_to_repertoire_grid,
-            EventSpec.partialdisplay_to_active_repertoire,
-            EventSpec.partialdisplay_to_repertoire_game_grid,
-            EventSpec.partialdisplay_to_partial_grid,
+    binding_labels = (
+            EventSpec.navigate_to_position_grid,
+            EventSpec.navigate_to_active_game,
+            EventSpec.navigate_to_game_grid,
+            EventSpec.navigate_to_repertoire_grid,
+            EventSpec.navigate_to_active_repertoire,
+            EventSpec.navigate_to_repertoire_game_grid,
+            EventSpec.navigate_to_partial_grid,
             EventSpec.partialdisplay_to_previous_partial,
             EventSpec.partialdisplay_to_next_partial,
-            EventSpec.partialdisplay_to_partial_game_grid,
-            EventSpec.partialdisplay_to_selection_rule_grid,
-            EventSpec.partialdisplay_to_active_selection_rule,
+            EventSpec.navigate_to_partial_game_grid,
+            EventSpec.navigate_to_selection_rule_grid,
+            EventSpec.navigate_to_active_selection_rule,
             EventSpec.tab_traverse_backward,
             EventSpec.tab_traverse_forward,
-            )])
+            )
 
     def __init__(self, sourceobject=None, **ka):
-        """Extend and link ChessQL statement to database.
-
-        sourceobject - link to database.
-
-        """
+        """Extend and link ChessQL statement to database."""
         super().__init__(**ka)
         self.blockchange = False
         if self.ui.base_partials.datasource:
@@ -93,134 +82,65 @@ class CQLDisplay(ExceptionHandler, Display):
         self.insertonly = sourceobject is None
         self.recalculate_after_edit = sourceobject
 
-    def _bind_for_board_navigation(self):
-        """Set bindings to navigate ChessQL statement on pointer click."""
-        self.bind_score_pointer_for_board_navigation(True)
-
-    def bind_for_widget_navigation(self):
-        """Set bindings to give focus to ChessQL statement on pointer click.
-        """
-        self.bind_score_pointer_for_widget_navigation(True)
-
-    def bind_off(self):
-        """Disable all bindings."""
+    @property
+    def ui_displayed_items(self):
+        return self.ui.partial_items
         
-        # Replicate structure of __bind_on for deleting bindings.
-        for sequence, function in (
-            (EventSpec.partialdisplay_to_partial_grid, ''),
-            (EventSpec.partialdisplay_to_previous_partial, ''),
-            (EventSpec.partialdisplay_to_next_partial, ''),
-            (EventSpec.partialdisplay_to_partial_game_grid, ''),
-            (EventSpec.partialdisplay_to_repertoire_grid, ''),
-            (EventSpec.partialdisplay_to_active_repertoire, ''),
-            (EventSpec.partialdisplay_to_repertoire_game_grid, ''),
-            (EventSpec.partialdisplay_to_position_grid, ''),
-            (EventSpec.partialdisplay_to_active_game, ''),
-            (EventSpec.partialdisplay_to_selection_rule_grid, ''),
-            (EventSpec.partialdisplay_to_active_selection_rule, ''),
-            (EventSpec.partialdisplay_to_game_grid, ''),
-            (EventSpec.export_from_partialdisplay, ''),
-            ):
-            if function:
-                function = self.try_event(function)
-            self.score.bind(sequence[0], function)
+    @property
+    def ui_configure_item_list_grid(self):
+        return self.ui.configure_partial_grid
+        
+    @property
+    def ui_set_item_name(self):
+        return self.ui._set_partial_name
+        
+    @property
+    def ui_set_find_item_games(self):
+        return self.ui._set_find_partial_name_games
 
-    # This method exists because the bindings cannot be done until all __init__
-    # methods in superclasses have been called.
-    def initialize_bindings(self):
-        """Initialize all bindings."""
-        self.__bind_on()
-        for sequence, function in (
+    def get_navigation_events(self):
+        """Return event description tuple for navigation from selection rule
+        statement."""
+        return (
+            (EventSpec.navigate_to_partial_grid,
+             self.set_focus_partial_grid),
+            (EventSpec.partialdisplay_to_previous_partial,
+             self.prior_item),
+            (EventSpec.partialdisplay_to_next_partial,
+             self.next_item),
+            (EventSpec.navigate_to_partial_game_grid,
+             self.set_focus_partial_game_grid),
+            (EventSpec.navigate_to_repertoire_grid,
+             self.set_focus_repertoire_grid),
+            (EventSpec.navigate_to_active_repertoire,
+             self.set_focus_repertoirepanel_item),
+            (EventSpec.navigate_to_repertoire_game_grid,
+             self.set_focus_repertoire_game_grid),
+            (EventSpec.navigate_to_position_grid,
+             self.set_focus_position_grid),
+            (EventSpec.navigate_to_active_game,
+             self.set_focus_gamepanel_item),
+            (EventSpec.navigate_to_selection_rule_grid,
+             self.set_focus_selection_rule_grid),
+            (EventSpec.navigate_to_active_selection_rule,
+             self.set_focus_selectionpanel_item),
+            (EventSpec.navigate_to_game_grid,
+             self.set_focus_game_grid),
+            (EventSpec.export_from_partialdisplay,
+             self.export_partial),
             (EventSpec.tab_traverse_forward,
              self.traverse_forward),
             (EventSpec.tab_traverse_backward,
              self.traverse_backward),
             (EventSpec.tab_traverse_round,
              self.traverse_round),
-            ):
-            if function:
-                function = self.try_event(function)
-            self.score.bind(sequence[0], function)
-
-    def bind_on(self):
-        """Enable all bindings."""
-        self.__bind_on()
-
-    def __bind_on(self):
-        """Enable all bindings."""
-
-        # Same bindings in initialize_bindings() and bind_on() in this class.
-        for sequence, function in (
-            ('<Control-ButtonPress-1>', ''),
-            ('<ButtonPress-1>', self.try_event(self.give_focus_to_widget)),
-            ('<ButtonPress-3>', self.try_event(self.popup_inactive_menu)),
-            ):
-            self.score.bind(sequence, function)
-        for sequence, function in (
-            (EventSpec.partialdisplay_to_partial_grid,
-             self.set_focus_partial_grid),
-            (EventSpec.partialdisplay_to_previous_partial,
-             self.prior_item),
-            (EventSpec.partialdisplay_to_next_partial,
-             self.next_item),
-            (EventSpec.partialdisplay_to_partial_game_grid,
-             self.set_focus_partial_game_grid),
-            (EventSpec.partialdisplay_to_repertoire_grid,
-             self.set_focus_repertoire_grid),
-            (EventSpec.partialdisplay_to_active_repertoire,
-             self.set_focus_repertoirepanel_item),
-            (EventSpec.partialdisplay_to_repertoire_game_grid,
-             self.set_focus_repertoire_game_grid),
-            (EventSpec.partialdisplay_to_position_grid,
-             self.set_focus_position_grid),
-            (EventSpec.partialdisplay_to_active_game,
-             self.set_focus_gamepanel_item),
-            (EventSpec.partialdisplay_to_selection_rule_grid,
-             self.set_focus_selection_rule_grid),
-            (EventSpec.partialdisplay_to_active_selection_rule,
-             self.set_focus_selectionpanel_item),
-            (EventSpec.partialdisplay_to_game_grid,
-             self.set_focus_game_grid),
-            (EventSpec.export_from_partialdisplay,
-             self.export_partial),
-            ):
-            if function:
-                function = self.try_event(function)
-            self.score.bind(sequence[0], function)
-
-    def _cycle_item(self, prior=False):
-        """Select next ChessQL statement on display."""
-        items = self.ui.partial_items
-        losefocus = items.active_item
-        losefocus.bind_for_widget_navigation()
-        items.cycle_active_item(prior=prior)
-        self.ui.configure_partial_grid()
-        gainfocus = items.active_item
-        gainfocus.refresh_game_list()
-        gainfocus._bind_for_board_navigation()
-        gainfocus.takefocus_widget.focus_set()
-        gainfocus.set_statusbar_text()
-
-    def give_focus_to_widget(self, event=None):
-        """Select ChessQL statement on display by mouse click."""
-        self.ui.set_bindings_on_item_losing_focus_by_pointer_click()
-        losefocus, gainfocus = self.ui.partial_items.give_focus_to_widget(
-            event.widget)
-        if losefocus is not gainfocus:
-            self.ui.configure_partial_grid()
-            self.score.after(
-                0,
-                func=self.try_command(self.ui._set_partial_name, self.score))
-            self.score.after(
-                0,
-                func=self.try_command(gainfocus.refresh_game_list, self.score))
-        return 'break'
+            )
 
     def delete_item_view(self, event=None):
         """Remove ChessQL statement item from screen."""
         self.ui.delete_position_view(self)
 
-    def insert_cql_statement_database(self, event=None):
+    def insert_item_database(self, event=None):
         """Add ChessQL statement to database."""
         if self.ui.partial_items.active_item is None:
             tkinter.messagebox.showinfo(
@@ -303,25 +223,6 @@ class CQLDisplay(ExceptionHandler, Display):
                              ' "',
                              updater.value.get_name_text(),
                              '" added to database.')))
-        
-    def next_item(self, event=None):
-        """Select next ChessQL statement display.
-
-        Call _next_cql_statement after 1 millisecond to allow message display
-
-        """
-        if self.ui.partial_items.count_items_in_stack() > 1:
-            self.ui._set_find_partial_name_games(0)
-            self.score.after(
-                1, func=self.try_command(self._next_cql_statement, self.score))
-
-    def _next_cql_statement(self):
-        """Generate next ChessQL statement display.
-
-        Call from next_item only.
-
-        """
-        self._cycle_item(prior=False)
 
     def on_game_change(self, instance):
         """Recalculate list of games for ChessQL statement after game update.
@@ -331,6 +232,16 @@ class CQLDisplay(ExceptionHandler, Display):
         """
         if self.sourceobject is not None:
             self._get_cql_statement_games_to_grid(self.cql_statement)#.match)
+        
+    def generate_popup_navigation_maps(self):
+        navigation_map = {k:v for k, v in self.get_navigation_events()}
+        local_map = {}
+        return navigation_map, local_map
+        
+    def create_primary_activity_popup(self):
+        popup = super().create_primary_activity_popup()
+        self.add_close_item_entry_to_popup(popup)
+        return popup
 
     def on_partial_change(self, instance):
         """Prevent update from self if instance refers to same record."""
@@ -437,39 +348,9 @@ class CQLDisplay(ExceptionHandler, Display):
                 # self.cql_statement is a CQLStatement instance.
                 # value is a ChessDBvaluePartial instance.
                 self.cql_statement = value
-                self.set_cql_statement()
+                self.set_and_tag_item_text()
                 self._get_cql_statement_games_to_grid(value)
 
-    def prior_item(self, event=None):
-        """Select previous ChessQL statement display.
-
-        Call _prior_cql_statement after 1 millisecond to allow message display
-
-        """
-        if self.ui.partial_items.count_items_in_stack() > 1:
-            self.ui._set_find_partial_name_games(-2)
-            self.score.after(
-                1, func=self.try_command(
-                    self._prior_cql_statement, self.score))
-
-    def _prior_cql_statement(self):
-        """Generate previous ChessQL statement display.
-
-        Call from prior_item only.
-
-        """
-        self._cycle_item(prior=True)
-
-    def set_insert_or_delete(self):
-        """Convert edit display to insert display.
-
-        ChessQL statements displayed for editing from a database are not closed
-        if the database is closed.  They are converted to insert displays and
-        can be used to add new records to databases opened later.
-        
-        """
-        self.sourceobject = None
-
     def get_text_for_statusbar(self):
         """"""
         return ''.join(
@@ -480,67 +361,15 @@ class CQLDisplay(ExceptionHandler, Display):
     def get_selection_text_for_statusbar(self):
         """"""
         return self.cql_statement.get_name_text()
-        
-    def bind_toplevel_navigation(self):
-        """Set bindings for popup menu for CQLDisplay instance."""
-        navigation_map = {
-            EventSpec.partialdisplay_to_position_grid[1]:
-            self.set_focus_position_grid,
-            EventSpec.partialdisplay_to_active_game[1]:
-            self.set_focus_gamepanel_item_command,
-            EventSpec.partialdisplay_to_game_grid[1]:
-            self.set_focus_game_grid,
-            EventSpec.partialdisplay_to_repertoire_grid[1]:
-            self.set_focus_repertoire_grid,
-            EventSpec.partialdisplay_to_active_repertoire[1]:
-            self.set_focus_repertoirepanel_item_command,
-            EventSpec.partialdisplay_to_repertoire_game_grid[1]:
-            self.set_focus_repertoire_game_grid,
-            EventSpec.partialdisplay_to_partial_grid[1]:
-            self.set_focus_partial_grid,
-            EventSpec.partialdisplay_to_partial_game_grid[1]:
-            self.set_focus_partial_game_grid,
-            EventSpec.partialdisplay_to_selection_rule_grid[1]:
-            self.set_focus_selection_rule_grid,
-            EventSpec.partialdisplay_to_active_selection_rule[1]:
-            self.set_focus_selectionpanel_item_command,
-            EventSpec.tab_traverse_backward[1]:
-            self.traverse_backward,
-            EventSpec.tab_traverse_forward[1]:
-            self.traverse_forward,
-            }
-        for nm, widget in (
-            ({EventSpec.partialdisplay_to_previous_partial[1]: self.prior_item,
-              EventSpec.partialdisplay_to_next_partial[1]: self.next_item,
-              },
-             self),
-            ):
-            nm.update(navigation_map)
-            widget.add_navigation_to_viewmode_popup(
-                bindings=nm, order=self.binding_labels)
 
     def set_game_list(self):
-        """"""
+        """Delegate to refresh_game_list via 'after(...) call."""
         self.panel.after(
             0, func=self.try_command(self.ui._set_partial_name, self.panel))
         self.panel.after(
             0,
             func=self.try_command(self.refresh_game_list, self.panel))
 
-    def traverse_backward(self, event=None):
-        """Give focus to previous widget type in traversal order."""
-        self.ui.give_focus_backward(self.ui.partial_items)
-        return 'break'
-
-    def traverse_forward(self, event=None):
-        """Give focus to next widget type in traversal order."""
-        self.ui.give_focus_forward(self.ui.partial_items)
-        return 'break'
-
-    def traverse_round(self, event=None):
-        """Give focus to next widget within active item in traversal order."""
-        return 'break'
-
     def _get_cql_statement_games_to_grid(self, statement):#match):
         """Populate Partial Position games grid with games selected by match.
 
@@ -596,96 +425,15 @@ class CQLDisplay(ExceptionHandler, Display):
                                      grid.datasource.not_implemented)))))
 
 
-class DatabaseCQLDisplay(CQLDisplay, CQL, DataNotify):
+class CQLDisplay(_CQLDisplay, DisplayText, ShowText, CQL, DataNotify):
     
     """Display ChessQL statement from database and allow delete and insert.
 
-    CQL provides the widget and CQLDisplay the database interface.
-    
+    Method delete_item_database allows records to be deleted from a database.
+
     """
 
-    def __init__(self, **ka):
-        """Create display widget and display ChessQL statement from database.
-
-        See superclasses for argument descriptions.
-
-        """
-        super().__init__(**ka)
-        self.initialize_bindings()
-
-    def bind_off(self):
-        """Disable all bindings."""
-        super().bind_off()
-        for sequence, function in (
-            (EventSpec.databasepartialdisplay_insert, ''),
-            (EventSpec.databasepartialdisplay_delete, ''),
-            (EventSpec.databasepartialdisplay_dismiss, ''),
-            ):
-            if function:
-                function = self.try_event(function)
-            self.score.bind(sequence[0], function)
-
-    # This method exists because the bindings cannot be done until all __init__
-    # methods in superclasses have been called.
-    def initialize_bindings(self):
-        """Initialize all bindings."""
-        super().initialize_bindings()
-
-        # Here because superclass order is CQLDisplay, CQL.
-        self.inactive_popup = tkinter.Menu(master=self.score, tearoff=False)
-
-        for function, accelerator in (
-            (self.set_focus_panel_item_command,
-             EventSpec.databasepartialdisplay_make_active),
-            (self.delete_item_view,
-             EventSpec.databasepartialdisplay_dismiss_inactive),
-            ):
-            self.inactive_popup.add_command(
-                label=accelerator[1],
-                command=self.try_command(function, self.inactive_popup),
-                accelerator=accelerator[2])
-        self.__bind_on()
-        for function, accelerator in (
-            (self.delete_item_view,
-             EventSpec.databasepartialdisplay_dismiss),
-            ):
-            self.viewmode_popup.add_command(
-                label=accelerator[1],
-                command=self.try_command(function, self.viewmode_popup),
-                accelerator=accelerator[2])
-        for function, accelerator in (
-            (self.insert_cql_statement_database,
-             EventSpec.databasepartialdisplay_insert),
-            (self.delete_cql_statement_database,
-             EventSpec.databasepartialdisplay_delete),
-            ):
-            self.viewmode_database_popup.add_command(
-                label=accelerator[1],
-                command=self.try_command(
-                    function, self.viewmode_database_popup),
-                accelerator=accelerator[2])
-        self.bind_toplevel_navigation()
-
-    def bind_on(self):
-        """Enable all bindings."""
-        super().bind_on()
-        self.__bind_on()
-
-    def __bind_on(self):
-        """Common to bind_on() and initialize_bindings()"""
-        for sequence, function in (
-            (EventSpec.databasepartialdisplay_insert,
-             self.insert_cql_statement_database),
-            (EventSpec.databasepartialdisplay_delete,
-             self.delete_cql_statement_database),
-            (EventSpec.databasepartialdisplay_dismiss,
-             self.delete_item_view),
-            ):
-            if function:
-                function = self.try_event(function)
-            self.score.bind(sequence[0], function)
-
-    def delete_cql_statement_database(self, event=None):
+    def delete_item_database(self, event=None):
         """Remove ChessQL statement from database."""
         if self.ui.database is None:
             tkinter.messagebox.showinfo(
@@ -753,95 +501,35 @@ class DatabaseCQLDisplay(CQLDisplay, CQL, DataNotify):
                              '" deleted from database.')))
 
 
-class DatabaseCQLInsert(CQLDisplay, CQLEdit, DataNotify):
+class CQLDisplayInsert(_CQLDisplay,
+                       ListGamesText,
+                       InsertText,
+                       ShowText,
+                       CQLEdit,
+                       DataNotify):
     
     """Display ChessQL statement from database allowing insert.
 
-    CQLEdit provides the widget and CQLDisplay the database
-    interface.
+    CQLEdit provides the widget and _CQLDisplay the database interface.
+
+    Listing games for ChessQL statements is different to selection rule
+    statements because selection rules share the listing area with the Game
+    and Index options to the Select menu.  Index opens up the 'Move to' and
+    Filter options too.  A definite user action is always required to generate
+    game lists for selection rules, in addition to navigating to (giving focus
+    to) the selection rule.  The area where ChessQL game lists are shown is
+    dedicated to ChessQL statements.
     
     """
-
-    def __init__(self, **ka):
-        """Create editor widget and display ChessQL statement from database.
-
-        See superclasses for argument descriptions.
-
-        """
-        super().__init__(**ka)
-        self.initialize_bindings()
-
-    def bind_off(self):
-        """Disable all bindings."""
-        super().bind_off()
-        for sequence, function in (
-            (EventSpec.databasepartialedit_show_game_list, ''),
-            (EventSpec.databasepartialedit_insert, ''),
-            (EventSpec.databasepartialedit_dismiss, ''),
-            ):
-            if function:
-                function = self.try_event(function)
-            self.score.bind(sequence[0], function)
-
-    # This method exists because the bindings cannot be done until all __init__
-    # methods in superclasses have been called.
-    def initialize_bindings(self):
-        """Initialize all bindings."""
-        super().initialize_bindings()
-
-        # Here because superclass order is CQLDisplay, CQLEdit.
-        self.inactive_popup = tkinter.Menu(master=self.score, tearoff=False)
-
-        for function, accelerator in (
-            (self.set_focus_panel_item_command,
-             EventSpec.databasepartialedit_make_active),
-            (self.delete_item_view,
-             EventSpec.databasepartialedit_dismiss_inactive),
-            ):
-            self.inactive_popup.add_command(
-                label=accelerator[1],
-                command=self.try_command(function, self.inactive_popup),
-                accelerator=accelerator[2])
-        self.__bind_on()
-        for function, accelerator in (
-            (self.process_and_set_cql_statement_list,
-             EventSpec.databasepartialedit_show_game_list),
-            (self.delete_item_view,
-             EventSpec.databasepartialedit_dismiss),
-            ):
-            self.viewmode_popup.add_command(
-                label=accelerator[1],
-                command=self.try_command(function, self.viewmode_popup),
-                accelerator=accelerator[2])
-        for function, accelerator in (
-            (self.insert_cql_statement_database,
-             EventSpec.databasepartialedit_insert),
-            ):
-            self.viewmode_database_popup.add_command(
-                label=accelerator[1],
-                command=self.try_command(
-                    function, self.viewmode_database_popup),
-                accelerator=accelerator[2])
-        self.bind_toplevel_navigation()
-
-    def bind_on(self):
-        """Enable all bindings."""
-        super().bind_on()
-        self.__bind_on()
-
-    def __bind_on(self):
-        """Common to bind_on() and initialize_bindings()"""
-        for sequence, function in (
-            (EventSpec.databasepartialedit_show_game_list,
-             self.process_and_set_cql_statement_list),
-            (EventSpec.databasepartialedit_insert,
-             self.insert_cql_statement_database),
-            (EventSpec.databasepartialedit_dismiss,
-             self.delete_item_view),
-            ):
-            if function:
-                function = self.try_event(function)
-            self.score.bind(sequence[0], function)
+    def get_list_games_events(self):
+        """Return tuple of event bindings to list games for ChessQL rule."""
+        return (
+            (EventSpec.display_list, self.process_and_set_cql_statement_list),
+            )
+        
+    def add_list_games_entry_to_popup(self, popup):
+        """Add option to list games for selection rule to popup."""
+        self.set_popup_bindings(popup, self.get_list_games_events())
 
     def process_and_set_cql_statement_list(self, event=None):
         """Display games with position matching edited ChessQL statement."""
@@ -883,82 +571,16 @@ class DatabaseCQLInsert(CQLDisplay, CQLEdit, DataNotify):
                 message=msg)
         return 'break'
 
-    def insert_char_to_right(self, char):
-        """"""
-        r = super().insert_char_to_right(char)
-        if r is None:
-            return None
-        elif r:
-            return True
-        else:
-            self.recalculate_after_edit = None
-            return None
 
-    def delete_char_left(self, event):
-        """"""
-        if not self.current:
-            return 'break'
-        self.recalculate_after_edit = None
-        return super().delete_char_left(event)
-
-    def delete_char_right(self, event):
-        """"""
-        if not self.current:
-            return 'break'
-        self.recalculate_after_edit = None
-        return super().delete_char_right(event)
-
-
-class DatabaseCQLEdit(DatabaseCQLInsert):
+class CQLDisplayEdit(EditText, CQLDisplayInsert):
     
     """Display ChessQL statement from database allowing edit and insert.
 
-    DatabaseCQLInsert adds insert to the database interface.
+    Method update_item_database allows records on the database to be amended.
     
     """
 
-    def bind_off(self):
-        """Disable all bindings."""
-        super().bind_off()
-        for sequence, function in (
-            (EventSpec.databasepartialedit_update, ''),
-            ):
-            if function:
-                function = self.try_event(function)
-            self.score.bind(sequence[0], function)
-
-    # This method exists because the bindings cannot be done until all __init__
-    # methods in superclasses have been called.
-    def initialize_bindings(self):
-        """Initialize all bindings."""
-        super().initialize_bindings()
-        self.__bind_on()
-        for function, accelerator in (
-            (self.update_cql_statement_database,
-             EventSpec.databasepartialedit_update),
-            ):
-            self.viewmode_database_popup.add_command(
-                label=accelerator[1],
-                command=self.try_command(
-                    function, self.viewmode_database_popup),
-                accelerator=accelerator[2])
-
-    def bind_on(self):
-        """Enable all bindings."""
-        super().bind_on()
-        self.__bind_on()
-
-    def __bind_on(self):
-        """Common to bind_on() and initialize_bindings()"""
-        for sequence, function in (
-            (EventSpec.databasepartialedit_update,
-             self.update_cql_statement_database),
-            ):
-            if function:
-                function = self.try_event(function)
-            self.score.bind(sequence[0], function)
-
-    def update_cql_statement_database(self, event=None):
+    def update_item_database(self, event=None):
         """Modify existing ChessQL statement record."""
         if self.ui.database is None:
             tkinter.messagebox.showinfo(
@@ -1057,61 +679,3 @@ class DatabaseCQLEdit(DatabaseCQLInsert):
                              ' "',
                              updater.value.get_name_text(),
                              '" amended on database.')))
-
-
-class CQLDialogue(ExceptionHandler):
-    
-    """Manage UI interaction with database for a displayed ChessQL statement.
-
-    Subclasses provide the widget to display the ChessQL statement.
-    
-    """
-
-    # Formally the same as GameDialogue in gamedisplay module.
-    # Follow whatever style of assignment is used there if this binding_labels
-    # needs to be non-empty.
-    binding_labels = tuple()
-
-    def initialize_bindings(self):
-        """Enable all bindings."""
-        self.bind_score_pointer_for_board_navigation(True)
-        self.bind_toplevel_navigation()
-        
-    def bind_toplevel_navigation(self):
-        """Set bindings for popup menu for CQLDialogue instance."""
-
-        # Formally the same as GameDialogue in gamedisplay module.
-        # The effect is add no bindings to an empty bindings map.
-        navigation_map = {}
-        for nm, widget in (
-            ({},
-             self),
-            ):
-            nm.update(navigation_map)
-            widget.add_navigation_to_viewmode_popup(
-                bindings=nm, order=self.binding_labels)
-
-
-class DialogueCQLDisplay(CQL, CQLDialogue):
-    
-    """Display a ChessQL statement from a database allowing delete and insert.
-    """
-
-    def __init__(self, **ka):
-        """Extend and link ChessQL statement to database.
-        """
-        super().__init__(**ka)
-        self.initialize_bindings()
-
-
-class DialogueCQLEdit(CQLEdit, CQLDialogue):
-    
-    """Display a ChessQL statement from a database allowing edit and insert.
-    """
-
-    def __init__(self, **ka):
-        """Extend and link ChessQL statement to database.
-
-        """
-        super().__init__(**ka)
-        self.initialize_bindings()
